# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/4/11 22:00
Desc: 上海黄金交易所-数据资讯-行情走势
https://www.sge.com.cn/sjzx/mrhq
上海黄金交易所-数据资讯-上海金基准价-历史数据
上海黄金交易所-数据资讯-上海银基准价-历史数据
"""

import pandas as pd
import requests

from akshare.utils.cons import headers


def spot_symbol_table_sge() -> pd.DataFrame:
    """
    上海黄金交易所-数据资讯-行情走势-品种表
    https://www.sge.com.cn/sjzx/mrhq
    :return: 品种表
    :rtype: pandas.DataFrame
    """
    temp_list = [
        "Au99.99",
        "Au99.95",
        "Au100g",
        "Pt99.95",
        "Ag(T+D)",
        "Au(T+D)",
        "mAu(T+D)",
        "Au(T+N1)",
        "Au(T+N2)",
        "Ag99.99",
        "iAu99.99",
        "Au99.5",
        "iAu100g",
        "iAu99.5",
        "PGC30g",
        "NYAuTN06",
        "NYAuTN12",
    ]
    temp_df = pd.DataFrame(temp_list)
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["序号", "品种"]
    temp_df["序号"] = temp_df["序号"] + 1
    return temp_df


def spot_quotations_sge(symbol: str = "Au99.99") -> pd.DataFrame:
    """
    上海黄金交易所-实时行情数据
    https://www.sge.com.cn/
    https://www.sge.com.cn/graph/quotations
    :param symbol: choice of {'Au99.99', 'Au99.95', 'Au100g', 'Pt99.95', 'Ag(T+D)', 'Au(T+D)',
    'mAu(T+D)', 'Au(T+N1)', 'Au(T+N2)', 'Ag99.99', 'iAu99.99', 'Au99.5', 'iAu100g',
    'iAu99.5', 'PGC30g', 'NYAuTN06', 'NYAuTN12'}; 可以通过 ak.spot_symbol_table_sge() 获取品种表
    :type symbol: str
    :return: 行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.sge.com.cn/graph/quotations"
    payload = {"instid": symbol}
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate, br, zstd",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "15",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.sge.com.cn",
        "Origin": "https://www.sge.com.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.sge.com.cn/",
        "sec-ch-ua": '"Google Chrome";v="107", "Chromium";v="107", "Not=A?Brand";v="24"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(
        {
            "品种": data_json["heyue"],
            "时间": data_json["times"],
            "现价": data_json["data"],
            "更新时间": data_json["delaystr"],
        }
    )
    temp_df["现价"] = pd.to_numeric(temp_df["现价"], errors="coerce")
    # 将更新时间中的时间部分提取出来
    update_time = temp_df["更新时间"].iloc[0].split()[1]
    # 将时间列转换为时间格式以便排序
    temp_df["时间"] = pd.to_datetime(temp_df["时间"], format="%H:%M").dt.time
    # 过滤掉大于等于更新时间的数据
    temp_df = temp_df[temp_df["时间"].astype(str) < update_time]
    # 按时间排序
    temp_df = temp_df.sort_values(by=["时间"])
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def spot_hist_sge(symbol: str = "Au99.99") -> pd.DataFrame:
    """
    上海黄金交易所-数据资讯-行情走势-历史数据
    https://www.sge.com.cn/sjzx/mrhq
    :param symbol: choice of {'Au99.99', 'Au99.95', 'Au100g', 'Pt99.95', 'Ag(T+D)', 'Au(T+D)',
    'mAu(T+D)', 'Au(T+N1)', 'Au(T+N2)', 'Ag99.99', 'iAu99.99', 'Au99.5', 'iAu100g', 'iAu99.5',
    'PGC30g', 'NYAuTN06', 'NYAuTN12'}; 可以通过 ak.spot_symbol_table_sge() 获取品种表
    :type symbol: str
    :return: 历史数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.sge.com.cn/graph/Dailyhq"
    payload = {"instid": symbol}
    headers = {
        "Accept": "text/html, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Length": "15",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.sge.com.cn",
        "Origin": "https://www.sge.com.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.sge.com.cn/sjzx/mrhq",
        "sec-ch-ua": '"Google Chrome";v="107", "Chromium";v="107", "Not=A?Brand";v="24"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["time"])
    temp_df.columns = [
        "date",
        "open",
        "close",
        "low",
        "high",
    ]

    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    return temp_df


def spot_golden_benchmark_sge() -> pd.DataFrame:
    """
    上海黄金交易所-数据资讯-上海金基准价-历史数据
    https://www.sge.com.cn/sjzx/jzj
    :return: 历史数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.sge.com.cn/graph/DayilyJzj"
    payload = {}
    r = requests.post(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["wp"])
    temp_df.columns = [
        "交易时间",
        "晚盘价",
    ]
    temp_df["交易时间"] = pd.to_datetime(temp_df["交易时间"], unit="ms").dt.date
    temp_zp_df = pd.DataFrame(data_json["zp"])
    temp_zp_df.columns = [
        "交易时间",
        "早盘价",
    ]
    temp_zp_df["交易时间"] = pd.to_datetime(
        temp_zp_df["交易时间"], unit="ms", errors="coerce"
    ).dt.date
    temp_df["早盘价"] = temp_zp_df["早盘价"]
    temp_df["晚盘价"] = pd.to_numeric(temp_df["晚盘价"], errors="coerce")
    temp_df["早盘价"] = pd.to_numeric(temp_df["早盘价"], errors="coerce")
    return temp_df


def spot_silver_benchmark_sge() -> pd.DataFrame:
    """
    上海黄金交易所-数据资讯-上海银基准价-历史数据
    https://www.sge.com.cn/sjzx/mrhq
    :return: 历史数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.sge.com.cn/graph/DayilyShsilverJzj"
    payload = {}
    r = requests.post(url, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["wp"])
    temp_df.columns = [
        "交易时间",
        "晚盘价",
    ]
    temp_df["交易时间"] = pd.to_datetime(temp_df["交易时间"], unit="ms").dt.date
    temp_zp_df = pd.DataFrame(data_json["zp"])
    temp_zp_df.columns = [
        "交易时间",
        "早盘价",
    ]
    temp_zp_df["交易时间"] = pd.to_datetime(
        temp_zp_df["交易时间"], unit="ms", errors="coerce"
    ).dt.date
    temp_df["早盘价"] = temp_zp_df["早盘价"]
    temp_df["晚盘价"] = pd.to_numeric(temp_df["晚盘价"], errors="coerce")
    temp_df["早盘价"] = pd.to_numeric(temp_df["早盘价"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    spot_symbol_table_sge_df = spot_symbol_table_sge()
    print(spot_symbol_table_sge_df)

    spot_hist_sge_df = spot_hist_sge(symbol="Au99.99")
    print(spot_hist_sge_df)

    spot_golden_benchmark_sge_df = spot_golden_benchmark_sge()
    print(spot_golden_benchmark_sge_df)

    spot_silver_benchmark_sge_df = spot_silver_benchmark_sge()
    print(spot_silver_benchmark_sge_df)

    for spot in spot_symbol_table_sge_df["品种"].tolist():
        spot_hist_sge_df = spot_hist_sge(symbol=spot)
        print(spot_hist_sge_df)

    spot_quotations_sge_df = spot_quotations_sge(symbol="Au99.99")
    print(spot_quotations_sge_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/12/25 16:46
Desc: 东方财富网-数据中心-新股申购-过会企业信息
https://data.eastmoney.com/xg/gh/default.html
"""

import json

import pandas as pd
import requests

from akshare.utils.cons import headers
from akshare.utils.tqdm import get_tqdm


def stock_ipo_review_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股申购-新股上会信息
    https://data.eastmoney.com/xg/gh/default.html
    :return: 新股上会信息
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "REVIEW_DATE,ORG_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_IPO_REVIEW",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
    }

    def parse_jsonp(text: str):
        """安全解析 JSONP 响应"""
        start = text.find('{')
        end = text.rfind('}')
        if start == -1 or end == -1 or end <= start:
            raise ValueError(f"无法找到 JSON 对象: {text[:200]}...")
        json_str = text[start:end + 1]
        return json.loads(json_str)

    # 首次请求获取总页数
    r = requests.get(url, params=params, headers=headers)
    r.raise_for_status()
    data_json = parse_jsonp(r.text)

    if "result" not in data_json or "pages" not in data_json["result"]:
        raise ValueError("响应格式异常，未找到 result.pages")

    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()

    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params["pageNumber"] = str(page)
        r = requests.get(url, params=params, headers=headers)
        r.raise_for_status()
        try:
            data_json = parse_jsonp(r.text)
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        except Exception as e:
            print(f"第 {page} 页解析失败: {e}")
            continue

    big_df.reset_index(drop=True, inplace=True)
    big_df.insert(0, "序号", big_df.index + 1)

    big_df.rename(columns={
        "ORG_NAME": "企业名称",
        "SECURITY_NAME_ABBR": "股票简称",
        "SECURITY_CODE": "股票代码",
        "TRADE_MARKET": "上市板块",
        "REVIEW_DATE": "上会日期",
        "REVIEW_STATE": "审核状态",
        "REVIEW_MEMBER": "发审委委员",
        "LEAD_UNDERWRITER": "主承销商",
        "ISSUE_NUM": "发行数量(股)",
        "FINANCE_AMT_UPPER": "拟融资额(元)",
        "NOTICE_DATE": "公告日期",
        "LISTING_DATE": "上市日期",
    }, inplace=True)

    big_df = big_df[[
        "序号",
        "企业名称",
        "股票简称",
        "股票代码",
        "上市板块",
        "上会日期",
        "审核状态",
        "发审委委员",
        "主承销商",
        "发行数量(股)",
        "拟融资额(元)",
        "公告日期",
        "上市日期",
    ]]

    # 日期处理
    date_cols = ["上会日期", "公告日期", "上市日期"]
    for col in date_cols:
        if col in big_df.columns:
            big_df[col] = pd.to_datetime(big_df[col], errors="coerce").dt.date

    # 数值格式化（可选）
    if "发行数量(股)" in big_df.columns:
        big_df["发行数量(股)"] = pd.to_numeric(big_df["发行数量(股)"], errors="coerce")
    if "拟融资额(元)" in big_df.columns:
        big_df["拟融资额(元)"] = pd.to_numeric(big_df["拟融资额(元)"], errors="coerce")

    return big_df


if __name__ == "__main__":
    stock_ipo_review_em_df = stock_ipo_review_em()
    print(stock_ipo_review_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/30 22:00
Desc: 中国-国家统计局-宏观数据
https://data.stats.gov.cn/easyquery.htm
"""

import time
from functools import lru_cache
from typing import Union, Literal, List, Dict

import jsonpath as jp
import numpy as np
import pandas as pd
import requests
import urllib3
from urllib3.exceptions import InsecureRequestWarning

# 忽略InsecureRequestWarning警告
urllib3.disable_warnings(InsecureRequestWarning)


@lru_cache
def _get_nbs_tree(idcode: str, dbcode: str) -> List[Dict]:
    """
    获取指标目录树
    :param idcode: 指标编码
    :param dbcode: 库编码
    :return:  json数据
    """
    url = "https://data.stats.gov.cn/easyquery.htm"
    params = {"id": idcode, "dbcode": dbcode, "wdcode": "zb", "m": "getTree"}
    r = requests.post(url, params=params, verify=False, allow_redirects=True)
    data_json = r.json()
    return data_json


@lru_cache
def _get_nbs_wds_tree(idcode: str, dbcode: str, rowcode: str) -> List[Dict]:
    """
    获取地区数据的可选指标目录树
    :param idcode: 指标编码
    :param dbcode: 库编码
    :param rowcode: 值为zb是返回地区的编码，值为reg时返回可选指标的编码
    :return:  json数据
    """
    url = "https://data.stats.gov.cn/easyquery.htm"
    params = {
        "m": "getOtherWds",
        "dbcode": dbcode,
        "rowcode": rowcode,
        "colcode": "sj",
        "wds": '[{"wdcode":"zb","valuecode":"%s"}]' % idcode,
        "k1": str(time.time_ns())[:13],
    }
    r = requests.post(url, params=params, verify=False, allow_redirects=True)
    data_json = r.json()
    data_json = data_json["returndata"][0]["nodes"]
    return data_json


def _get_code_from_nbs_tree(tree: List[Dict], name: str, target: str = "id") -> str:
    """
    根据指标名称从目录树中获取target编码
    :param tree: 目录树
    :param name: 指标名称
    :param target: 指标编码属性名
    :return: 指标编码
    """
    expr = f'$[?(@.name == "{name}")].{target}'
    ret = jp.jsonpath(tree, expr)
    if ret is False:
        raise ValueError("Please check if the data path or indicator is correct.")
    return ret[0]


def macro_china_nbs_nation(
    kind: Literal["月度数据", "季度数据", "年度数据"], path: str, period: str = "LAST10"
) -> pd.DataFrame:
    """
    国家统计局全国数据通用接口
    https://data.stats.gov.cn/easyquery.htm
    :param kind: 数据类别
    :param path: 数据路径
    :param period: 时间区间，例如'LAST10', '2016-2023', '2016-'等
    :return: 国家统计局统计数据
    :rtype: pandas.DataFrame
    """
    # 获取dbcode
    kind_code = {"月度数据": "hgyd", "季度数据": "hgjd", "年度数据": "hgnd"}
    dbcode = kind_code[kind]

    # 获取最终id
    parent_tree = _get_nbs_tree("zb", dbcode)
    path_split = path.replace(" ", "").split(">")
    indicator_id = _get_code_from_nbs_tree(parent_tree, path_split[0])
    path_split.pop(0)
    while path_split:
        temp_tree = _get_nbs_tree(indicator_id, dbcode)
        indicator_id = _get_code_from_nbs_tree(temp_tree, path_split[0])
        path_split.pop(0)

    # 请求数据
    url = "https://data.stats.gov.cn/easyquery.htm"
    params = {
        "m": "QueryData",
        "dbcode": dbcode,
        "rowcode": "zb",
        "colcode": "sj",
        "wds": "[]",
        "dfwds": '[{"wdcode":"zb","valuecode":"%s"}, '
        '{"wdcode":"sj","valuecode":"%s"}]' % (indicator_id, period),
        "k1": str(time.time_ns())[:13],
    }
    r = requests.get(url, params=params, verify=False, allow_redirects=True)
    data_json = r.json()

    # 整理为dataframe
    temp_df = pd.DataFrame(data_json["returndata"]["datanodes"])
    temp_df["data"] = temp_df["data"].apply(
        lambda x: x["data"] if x["hasdata"] else None
    )

    wdnodes = data_json["returndata"]["wdnodes"]
    wn_df_list = []
    for wn in wdnodes:
        wn_df_list.append(
            pd.DataFrame(wn["nodes"])
            .assign(
                funit=lambda df: df["unit"].apply(lambda x: "(" + x + ")" if x else x)
            )
            .assign(fname=lambda df: df["cname"] + df["funit"]),
        )

    row_name, column_name = (
        wn_df_list[0]["fname"],
        wn_df_list[1]["fname"],
    )

    data_ndarray = np.reshape(temp_df["data"], (len(row_name), len(column_name)))
    data_df = pd.DataFrame(data=data_ndarray, columns=column_name, index=row_name)
    data_df.index.name = None
    data_df.columns.name = None

    return data_df


def macro_china_nbs_region(
    kind: Literal[
        "分省月度数据",
        "分省季度数据",
        "分省年度数据",
        "主要城市月度价格",
        "主要城市年度数据",
        "港澳台月度数据",
        "港澳台年度数据",
    ],
    path: str,
    indicator: Union[str, None],
    region: Union[str, None] = None,
    period: str = "LAST10",
) -> pd.DataFrame:
    """
    国家统计局地区数据通用接口
    https://data.stats.gov.cn/easyquery.htm
    :param kind: 数据类别
    :param path: 数据路径
    :param indicator: 指定指标
    :param region:  指定地区 当指定region时，将symbol设为None可以同时获得所有可选指标的值
    :param period: 时间区间，例如'LAST10', '2016-2023', '2016-'等
    :return: 国家统计局统计数据
    :rtype: pandas.DataFrame
    """
    if indicator is None and region is None:
        raise AssertionError("The indicator and region parameters cannot both be None.")

    # 获取dbcode
    kind_dict = {
        "分省月度数据": "fsyd",
        "分省季度数据": "fsjd",
        "分省年度数据": "fsnd",
        "主要城市月度价格": "csyd",
        "主要城市年度数据": "csnd",
        "港澳台月度数据": "gatyd",
        "港澳台年度数据": "gatnd",
    }
    dbcode = kind_dict[kind]

    # 获取最终id
    parent_tree = _get_nbs_tree("zb", dbcode)
    path_split = path.replace(" ", "").split(">")
    indicator_id = _get_code_from_nbs_tree(parent_tree, path_split[0])
    path_split.pop(0)
    while path_split:
        temp_tree = _get_nbs_tree(indicator_id, dbcode)
        indicator_id = _get_code_from_nbs_tree(temp_tree, path_split[0])
        path_split.pop(0)

    # 参数设定
    if region is None:
        indicator_tree = _get_nbs_wds_tree(indicator_id, dbcode, "reg")
        indicator_id = _get_code_from_nbs_tree(indicator_tree, indicator, target="code")
        rowcode = "reg"
        colcode = "sj"
        wds = '[{"wdcode":"zb","valuecode":"%s"}]' % indicator_id
        dfwds = '[{"wdcode":"sj","valuecode":"%s"}]' % period
    else:
        if indicator is not None:
            indicator_tree = _get_nbs_wds_tree(indicator_id, dbcode, "reg")
            indicator_id = _get_code_from_nbs_tree(
                indicator_tree, indicator, target="code"
            )
        region_tree = _get_nbs_wds_tree(indicator_id, dbcode, "zb")
        region_id = _get_code_from_nbs_tree(region_tree, region, target="code")
        rowcode = "zb"
        colcode = "sj"
        wds = '[{"wdcode":"reg","valuecode":"%s"}]' % region_id
        dfwds = (
            '[{"wdcode":"zb","valuecode":"%s"}, '
            '{"wdcode":"sj","valuecode":"%s"}]' % (indicator_id, period)
        )

    # 请求数据
    url = "https://data.stats.gov.cn/easyquery.htm"
    params = {
        "m": "QueryData",
        "dbcode": dbcode,
        "rowcode": rowcode,
        "colcode": colcode,
        "wds": wds,
        "dfwds": dfwds,
        "k1": str(time.time_ns())[:13],
    }
    r = requests.get(url, params=params, verify=False, allow_redirects=True)
    data_json = r.json()

    # 整理为dataframe
    temp_df = pd.DataFrame(data_json["returndata"]["datanodes"])
    temp_df["data"] = temp_df["data"].apply(
        lambda x: x["data"] if x["hasdata"] else None
    )

    wdnodes = data_json["returndata"]["wdnodes"]
    wn_df_list = []
    for wn in wdnodes:
        wn_df_list.append(
            pd.DataFrame(wn["nodes"])
            .assign(
                funit=lambda df: df["unit"].apply(lambda x: "(" + x + ")" if x else x)
            )
            .assign(fname=lambda df: df["cname"] + df["funit"]),
        )

    if region is None:
        row_name, column_name = wn_df_list[1]["fname"], wn_df_list[2]["fname"]
        title_name = wn_df_list[0]["fname"][0]
    else:
        row_name, column_name = wn_df_list[0]["fname"], wn_df_list[2]["fname"]
        title_name = wn_df_list[1]["fname"][0]

    data_ndarray = np.reshape(temp_df["data"], (len(row_name), len(column_name)))
    data_df = pd.DataFrame(data=data_ndarray, columns=column_name, index=row_name)
    data_df.index.name = None
    data_df.columns.name = title_name

    return data_df


if __name__ == "__main__":
    macro_china_nbs_nation_df = macro_china_nbs_nation(
        kind="月度数据",
        path="工业 > 工业分大类行业出口交货值(2018-至今) > 废弃资源综合利用业",
        period="LAST5",
    )
    print(macro_china_nbs_nation_df)

    macro_china_nbs_region_df = macro_china_nbs_region(
        kind="分省季度数据",
        path="人民生活 > 居民人均可支配收入",
        period="2018-2022",
        indicator=None,
        region="北京市",
    )
    print(macro_china_nbs_region_df)

    macro_china_nbs_region_df = macro_china_nbs_region(
        kind="分省季度数据",
        path="国民经济核算 > 地区生产总值",
        period="2018-",
        indicator="地区生产总值_累计值(亿元)",
    )
    print(macro_china_nbs_region_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/12/1 22:00
Desc: 百度股市通-经济数据
https://gushitong.baidu.com/calendar
"""
import math
import re

import pandas as pd
from curl_cffi import requests


def _get_baidu_cookie(headers: dict) -> str:
    """
    安全获取百度股市通所需的Cookie
    :param headers: 基础请求头
    :return: 格式化的Cookie字符串
    :raises ValueError: 当无法获取必要Cookie时
    :raises ConnectionError: 网络请求失败时
    """
    try:
        # 使用Session保持Cookie上下文
        with requests.Session() as session:
            session.headers.update(headers)

            # 第一步：获取基础Cookie (BAIDUID系列)
            resp1 = session.get(
                "https://gushitong.baidu.com/calendar",
                impersonate="chrome110",
                timeout=10
            )
            resp1.raise_for_status()

            # 验证必要Cookie
            baiduid = resp1.cookies.get("BAIDUID")
            baiduid_bfess = resp1.cookies.get("BAIDUID_BFESS")
            if not all([baiduid, baiduid_bfess]):
                raise ValueError("Missing BAIDUID cookies in first response")

            # 第二步：提取并请求hm.js
            hm_pattern = r'https://hm\.baidu\.com/hm\.js\?\w+'
            hm_match = re.search(hm_pattern, resp1.text)
            if not hm_match:
                # 尝试备用正则模式
                hm_match = re.search(r'//hm\.baidu\.com/hm\.js\?\w+', resp1.text)
                if not hm_match:
                    raise ValueError("Failed to extract hm.js URL from response")

            hm_url = "https:" + hm_match.group() if hm_match.group().startswith("//") else hm_match.group()

            # 第二步请求 (自动携带第一步的Cookie)
            resp2 = session.get(hm_url, impersonate="chrome110", timeout=10)
            resp2.raise_for_status()

            # 验证必要Cookie
            hmac_count = resp2.cookies.get("HMACCOUNT")
            hmac_count_bfess = resp2.cookies.get("HMACCOUNT_BFESS")
            if not all([hmac_count, hmac_count_bfess]):
                raise ValueError("Missing HMACCOUNT cookies in second response")

            # 安全拼接Cookie
            return (
                f"BAIDUID={baiduid}; "
                f"BAIDUID_BFESS={baiduid_bfess}; "
                f"HMACCOUNT={hmac_count}; "
                f"HMACCOUNT_BFESS={hmac_count_bfess}"
            )

    except requests.exceptions.RequestException as e:
        raise ConnectionError(f"Network request failed: {str(e)}") from e
    except re.error as e:
        raise ValueError(f"Regex pattern error: {str(e)}") from e


def _baidu_finance_calendar(
    date: str,
    cate: str,
    process_func,
    cookie: str = None
) -> pd.DataFrame:
    """
    百度股市通日历数据基础函数（支持分页）
    :param date: 查询日期 (格式: YYYYMMDD)
    :param cate: 数据类别 ("economic_data" 或 "notify_suspend")
    :param process_func: 数据处理函数
    :param cookie: cookie
    :return: 处理后的DataFrame
    """
    # 日期格式转换
    formatted_date = "-".join([date[:4], date[4:6], date[6:]])

    # 构建请求参数
    base_params = {
        "start_date": formatted_date,
        "end_date": formatted_date,
        "pn": "0",
        "rn": "100",  # 每页100条
        "cate": cate,
        "finClientType": "pc",
    }

    # 构建请求头
    headers = {
        "accept": "application/vnd.finance-web.v1+json",
        "accept-encoding": "gzip, deflate, br, zstd",
        "accept-language": "en,zh-CN;q=0.9,zh;q=0.8",
        "cache-control": "no-cache",
        "origin": "https://gushitong.baidu.com",
        "pragma": "no-cache",
        "priority": "u=1, i",
        "referer": "https://gushitong.baidu.com/",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/142.0.0.0 Safari/537.36"
    }

    # 在_baidu_finance_calendar函数中替换原代码块
    if cookie is None:
        try:
            cookie = _get_baidu_cookie(headers.copy())  # 保护原始headers
        except Exception as e:
            # 可降级处理或保留原始行为
            raise RuntimeError(f"Failed to obtain Baidu cookies: {str(e)}") from e
    headers["cookie"] = cookie

    url = "https://finance.pae.baidu.com/sapi/v1/financecalendar"
    big_df = pd.DataFrame()

    # 获取指定日期的总记录数
    target_date = formatted_date
    total_records = 0

    # 第一次请求
    params = base_params.copy()
    response = requests.get(url=url, params=params, headers=headers)
    response.raise_for_status()
    data_json = response.json()

    # 从JSON中提取指定日期的总记录数
    if "Result" in data_json and "calendarInfo" in data_json["Result"]:
        calendar_info = data_json["Result"]["calendarInfo"]

        # 查找目标日期的记录
        for item in calendar_info:
            if item.get("date") == target_date:
                total_records = item.get("total", 0)
                break

    # 计算总页数 (每页100条)
    total_pages = math.ceil(total_records / 100) if total_records > 0 else 1

    # 处理所有页码
    for page in range(total_pages):
        if page > 0:  # 第一页已在前面获取
            params = base_params.copy()
            params["pn"] = str(page)
            response = requests.get(url=url, params=params, headers=headers)
            response.raise_for_status()
            data_json = response.json()

        # 提取并处理指定日期的数据
        if "Result" in data_json and "calendarInfo" in data_json["Result"]:
            for item in data_json["Result"]["calendarInfo"]:
                if item.get("date") == target_date and item.get("list"):
                    processed_df = process_func(item["list"])
                    big_df = pd.concat([big_df, processed_df], ignore_index=True)

    return big_df


def _process_economic_data(data_list: list) -> pd.DataFrame:
    """处理经济数据"""
    if not data_list:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_list)
    rename_dict = {
        "date": "日期",
        "time": "时间",
        "title": "事件",
        "star": "重要性",
        "formerVal": "前值",
        "pubVal": "公布",
        "region": "地区",
        "indicateVal": "预期",
        "country": "国家",
        "timePeriod": "统计周期"
    }
    temp_df.rename(columns=rename_dict, inplace=True)
    required_cols = ["公布", "预期", "前值", "重要性"]
    for col in required_cols:
        if col not in temp_df.columns:
            temp_df[col] = None
    available_cols = []
    for col in ["日期", "时间", "国家", "地区", "事件", "统计周期", "公布", "预期", "前值", "重要性"]:
        if col in temp_df.columns:
            available_cols.append(col)
    if available_cols:
        temp_df = temp_df[available_cols]
    for col in ["公布", "预期", "前值", "重要性"]:
        if col in temp_df.columns:
            temp_df[col] = pd.to_numeric(temp_df[col], errors="coerce")
    if "日期" in temp_df.columns:
        temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date

    return temp_df


def _process_suspend_data(data_list: list) -> pd.DataFrame:
    """处理停复牌数据 - 根据实际JSON结构精确修正"""
    if not data_list:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_list)
    rename_dict = {
        "code": "股票代码",
        "name": "股票简称",
        "exchange": "交易所代码",
        "start": "停牌时间",
        "reason": "停牌事项说明",
        "marketValue": "市值",
        "date": "公告日期",
        "time": "公告时间",
        "type": "证券类型",
        "market": "市场类型",
        "isSkip": "是否跳过",
        "end": "复牌时间"
    }
    temp_df.rename(columns=rename_dict, inplace=True)
    if "复牌时间" not in temp_df.columns:
        temp_df['复牌时间'] = '-'
    temp_df = temp_df[[
        "股票代码",
        "股票简称",
        "交易所代码",
        "停牌时间",
        "复牌时间",
        "停牌事项说明",
        "市值",
        "公告日期",
        "公告时间",
        "证券类型",
        "市场类型",
        "是否跳过"
    ]]
    return temp_df


def news_economic_baidu(date: str = "20251126", cookie: str = None) -> pd.DataFrame:
    """
    百度股市通-经济数据
    https://gushitong.baidu.com/calendar
    :param date: 查询日期 (格式: YYYYMMDD)
    :param cookie: cookie
    :return: 经济数据DataFrame
    """
    return _baidu_finance_calendar(
        date=date,
        cate="economic_data",
        process_func=_process_economic_data,
        cookie=cookie
    )


def news_trade_notify_suspend_baidu(date: str = "20251126", cookie: str = None) -> pd.DataFrame:
    """
    百度股市通-交易提醒-停复牌
    https://gushitong.baidu.com/calendar
    :param date: 查询日期 (格式: YYYYMMDD)
    :param cookie: cookie
    :return: 停复牌数据DataFrame
    """
    return _baidu_finance_calendar(
        date=date,
        cate="notify_suspend",
        process_func=_process_suspend_data,
        cookie=cookie
    )


def _process_dividend_data(data_list: list) -> pd.DataFrame:
    """处理分红派息数据"""
    if not data_list:
        return pd.DataFrame()

    temp_df = pd.DataFrame(data_list)

    # 字段映射
    rename_dict = {
        "code": "股票代码",
        "market": "-",  # 这个字段在最终结果中会被删除
        "exchange": "交易所",
        "name": "股票简称",
        "diviDate": "除权日",
        "date": "报告期",
        "diviCash": "分红",
        "shareDivide": "送股",
        "transfer": "转增",
        "physical": "实物",
    }
    temp_df.rename(columns=rename_dict, inplace=True)

    # 确保必要列存在
    if "实物" not in temp_df.columns:
        temp_df["实物"] = "-"
    if "送股" not in temp_df.columns:
        temp_df["送股"] = "-"
    if "转增" not in temp_df.columns:
        temp_df["转增"] = "-"

    # 选择需要的列
    temp_df = temp_df[
        [
            "股票代码",
            "除权日",
            "分红",
            "送股",
            "转增",
            "实物",
            "交易所",
            "股票简称",
            "报告期",
        ]
    ]

    # 日期格式转换
    if "除权日" in temp_df.columns:
        temp_df["除权日"] = pd.to_datetime(temp_df["除权日"], errors="coerce").dt.date
    if "报告期" in temp_df.columns:
        temp_df["报告期"] = pd.to_datetime(temp_df["报告期"], errors="coerce").dt.date

    return temp_df


def news_trade_notify_dividend_baidu(date: str = "20251126", cookie: str = None) -> pd.DataFrame:
    """
    百度股市通-交易提醒-分红派息
    https://gushitong.baidu.com/calendar
    :param date: 查询日期 (格式: YYYYMMDD)
    :param cookie: cookie
    :return: 交易提醒-分红派息DataFrame
    """
    return _baidu_finance_calendar(
        date=date,
        cate="notify_divide",
        process_func=_process_dividend_data,
        cookie=cookie
    )


def _process_report_data(data_list: list) -> pd.DataFrame:
    """处理财报发行数据 - 根据实际JSON结构精确修正"""
    if not data_list:
        return pd.DataFrame()

    # 创建DataFrame
    temp_df = pd.DataFrame(data_list)

    # 精确字段映射 (根据提供的JSON结构)
    rename_dict = {
        "code": "股票代码",
        "name": "股票简称",
        "exchange": "交易所",
        "reportType": "财报类型",
        "time": "发布时间",
        "marketValue": "市值",
        "capitalization": "总市值",
        "date": "发布日期"
    }
    temp_df.rename(columns=rename_dict, inplace=True)

    # 确保必要列存在
    if "财报类型" not in temp_df.columns:
        temp_df["财报类型"] = "-"
    if "发布时间" not in temp_df.columns:
        temp_df["发布时间"] = "-"
    if "市值" not in temp_df.columns and "总市值" in temp_df.columns:
        temp_df["市值"] = temp_df["总市值"]

    # 选择并排序列
    available_cols = []
    for col in ["股票代码", "股票简称", "交易所", "财报类型", "发布时间", "市值", "发布日期"]:
        if col in temp_df.columns:
            available_cols.append(col)

    if available_cols:
        temp_df = temp_df[available_cols]
    else:
        # 如果没有匹配的列，返回空DataFrame
        return pd.DataFrame()

    # 类型转换
    if "市值" in temp_df.columns:
        temp_df["市值"] = pd.to_numeric(temp_df["市值"], errors="coerce")

    if "发布日期" in temp_df.columns:
        temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date

    return temp_df


def news_report_time_baidu(date: str = "20251126", cookie: str = None) -> pd.DataFrame:
    """
    百度股市通-财报发行
    https://gushitong.baidu.com/calendar
    :param date: 查询日期 (格式: YYYYMMDD)
    :param cookie: cookie
    :return: 财报发行DataFrame
    """
    return _baidu_finance_calendar(
        date=date,
        cate="report_time",
        process_func=_process_report_data,
        cookie=cookie
    )


if __name__ == "__main__":
    news_economic_baidu_df = news_economic_baidu(date="20251126")
    print(news_economic_baidu_df)

    news_trade_notify_suspend_baidu_df = news_trade_notify_suspend_baidu(
        date="20251126"
    )
    print(news_trade_notify_suspend_baidu_df)

    news_trade_notify_dividend_baidu_df = news_trade_notify_dividend_baidu(
        date="20251126"
    )
    print(news_trade_notify_dividend_baidu_df)

    news_report_time_baidu_df = news_report_time_baidu(date="20251126")
    print(news_report_time_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/4 18:20
Desc: 东方财富网-数据中心-特色数据-千股千评
https://data.eastmoney.com/stockcomment/
"""

import json
import re
import time

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_comment_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-千股千评
    https://data.eastmoney.com/stockcomment/
    :return: 千股千评数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "SECURITY_CODE",
        "sortTypes": "1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_DMSK_TS_STOCKNEW",
        "quoteColumns": "f2~01~SECURITY_CODE~CLOSE_PRICE,f8~01~SECURITY_CODE~TURNOVERRATE,"
                        "f3~01~SECURITY_CODE~CHANGE_RATE,f9~01~SECURITY_CODE~PE_DYNAMIC",
        "columns": "ALL",
        "filter": "",
        "token": "894050c76af8597a853f5b408b759f5d",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1
    big_df.columns = [
        "序号",
        "-",
        "代码",
        "-",
        "交易日",
        "名称",
        "-",
        "-",
        "-",
        "最新价",
        "涨跌幅",
        "-",
        "换手率",
        "主力成本",
        "市盈率",
        "-",
        "-",
        "机构参与度",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "综合得分",
        "上升",
        "目前排名",
        "关注指数",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "换手率",
            "市盈率",
            "主力成本",
            "机构参与度",
            "综合得分",
            "上升",
            "目前排名",
            "关注指数",
            "交易日",
        ]
    ]

    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["换手率"] = pd.to_numeric(big_df["换手率"], errors="coerce")
    big_df["市盈率"] = pd.to_numeric(big_df["市盈率"], errors="coerce")
    big_df["主力成本"] = pd.to_numeric(big_df["主力成本"], errors="coerce")
    big_df["机构参与度"] = pd.to_numeric(big_df["机构参与度"], errors="coerce")
    big_df["综合得分"] = pd.to_numeric(big_df["综合得分"], errors="coerce")
    big_df["上升"] = pd.to_numeric(big_df["上升"], errors="coerce")
    big_df["目前排名"] = pd.to_numeric(big_df["目前排名"], errors="coerce")
    big_df["关注指数"] = pd.to_numeric(big_df["关注指数"], errors="coerce")
    big_df["交易日"] = pd.to_datetime(big_df["交易日"], errors="coerce").dt.date
    return big_df


def stock_comment_detail_zlkp_jgcyd_em(symbol: str = "600000") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-千股千评-主力控盘-机构参与度
    https://data.eastmoney.com/stockcomment/stock/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 主力控盘-机构参与度
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_DMSK_TS_STOCKEVALUATE",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df = temp_df[["TRADE_DATE", "ORG_PARTICIPATE"]]
    temp_df.columns = ["交易日", "机构参与度"]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    temp_df.sort_values(["交易日"], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["机构参与度"] = pd.to_numeric(temp_df["机构参与度"], errors="coerce") * 100
    return temp_df


def stock_comment_detail_zhpj_lspf_em(symbol: str = "600000") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-千股千评-综合评价-历史评分
    https://data.eastmoney.com/stockcomment/stock/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 综合评价-历史评分
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "filter": f'(SECURITY_CODE="{symbol}")',
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "reportName": "RPT_STOCK_HISTORYMARK",
        "sortColumns": "DIAGNOSE_DATE",
        "sortTypes": "1",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "TOTAL_SCORE": "评分",
            "DIAGNOSE_DATE": "交易日",
        },
        inplace=True,
    )
    temp_df = temp_df[["交易日", "评分"]]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    temp_df.sort_values(by=["交易日"], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["评分"] = pd.to_numeric(temp_df["评分"], errors="coerce")
    return temp_df


def stock_comment_detail_scrd_focus_em(symbol: str = "600000") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-千股千评-市场热度-用户关注指数
    https://data.eastmoney.com/stockcomment/stock/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 市场热度-用户关注指数
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "filter": f'(SECURITY_CODE="{symbol}")',
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "reportName": "RPT_STOCK_MARKETFOCUS",
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "30",
    }
    r = requests.get(url=url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "MARKET_FOCUS": "用户关注指数",
            "TRADE_DATE": "交易日",
        },
        inplace=True,
    )
    temp_df = temp_df[["交易日", "用户关注指数"]]
    temp_df["交易日"] = pd.to_datetime(temp_df["交易日"], errors="coerce").dt.date
    temp_df.sort_values(by=["交易日"], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["用户关注指数"] = pd.to_numeric(temp_df["用户关注指数"], errors="coerce")
    return temp_df


def stock_comment_detail_scrd_desire_em(
    symbol: str = "600000",
) -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-千股千评-市场热度-市场参与意愿
    https://data.eastmoney.com/stockcomment/stock/600000.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 市场热度-市场参与意愿
    :rtype: pandas.DataFrame
    """
    url = f"https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "callback": f"jQuery11230899775623921407_{int(time.time() * 1000)}",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "reportName": "RPT_STOCK_PARTICIPATION",
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "30",
        "_": int(time.time() * 1000)
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/91.0.4472.124 Safari/537.36",
        "Referer": "https://data.eastmoney.com/",
        "Accept": "*/*",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    }
    r = requests.get(url, params=params, headers=headers)
    jsonp_data = r.text
    json_str = re.search(r'\((.*)\)', jsonp_data).group(1)
    data_json = json.loads(json_str)
    data_list = data_json['result']['data']
    temp_df = pd.DataFrame(data_list)
    temp_df['TRADE_DATE'] = pd.to_datetime(temp_df['TRADE_DATE'], errors="coerce").dt.date
    column_mapping = {
        'SECURITY_INNER_CODE': '内部代码',
        'SECURITY_CODE': '股票代码',
        'TRADE_DATE': '交易日期',
        'PARTICIPATION_WISH': '参与意愿',
        'PARTICIPATION_WISH_5DAYS': '5日平均参与意愿',
        'PARTICIPATION_WISH_CHANGE': '参与意愿变化',
        'PARTICIPATION_WISH_5DAYSCHANGE': '5日平均变化'
    }
    temp_df = temp_df.rename(columns=column_mapping)
    column_order = [
        '交易日期', '股票代码', '内部代码',
        '参与意愿', '5日平均参与意愿',
        '参与意愿变化', '5日平均变化'
    ]
    temp_df = temp_df[column_order]
    del temp_df['内部代码']
    temp_df.sort_values(by=['交易日期'], ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    stock_comment_em_df = stock_comment_em()
    print(stock_comment_em_df)

    stock_comment_detail_zlkp_jgcyd_em_df = stock_comment_detail_zlkp_jgcyd_em(
        symbol="600000"
    )
    print(stock_comment_detail_zlkp_jgcyd_em_df)

    stock_comment_detail_zhpj_lspf_em_df = stock_comment_detail_zhpj_lspf_em(
        symbol="600000"
    )
    print(stock_comment_detail_zhpj_lspf_em_df)

    stock_comment_detail_scrd_focus_em_df = stock_comment_detail_scrd_focus_em(
        symbol="600000"
    )
    print(stock_comment_detail_scrd_focus_em_df)

    stock_comment_detail_scrd_desire_em_df = stock_comment_detail_scrd_desire_em(
        symbol="600000"
    )
    print(stock_comment_detail_scrd_desire_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/3 16:36
Desc: 金十数据-其他-加密货币实时行情
"""

from datetime import datetime

import pandas as pd
import requests


def crypto_js_spot() -> pd.DataFrame:
    """
    主流加密货币的实时行情数据, 一次请求返回具体某一时刻行情数据
    https://datacenter.jin10.com/reportType/dc_bitcoin_current
    :return: pandas.DataFrame
    """
    url = "https://datacenter-api.jin10.com/crypto_currency/list"
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }
    r = requests.get(url, headers=headers)
    data_json = r.json()
    data_df = pd.DataFrame(data_json["data"])
    data_df["reported_at"] = pd.to_datetime(data_df["reported_at"])
    data_df.columns = [
        "市场",
        "交易品种",
        "最近报价",
        "涨跌额",
        "涨跌幅",
        "24小时最高",
        "24小时最低",
        "24小时成交量",
        "更新时间",
    ]
    data_df["最近报价"] = pd.to_numeric(data_df["最近报价"], errors="coerce")
    data_df["涨跌额"] = pd.to_numeric(data_df["涨跌额"], errors="coerce")
    data_df["涨跌幅"] = pd.to_numeric(data_df["涨跌幅"], errors="coerce")
    data_df["24小时最高"] = pd.to_numeric(data_df["24小时最高"], errors="coerce")
    data_df["24小时最低"] = pd.to_numeric(data_df["24小时最低"], errors="coerce")
    data_df["24小时成交量"] = pd.to_numeric(data_df["24小时成交量"], errors="coerce")
    data_df["更新时间"] = data_df["更新时间"].astype(str)
    return data_df


def macro_fx_sentiment(
    start_date: str = "20221011", end_date: str = "20221017"
) -> pd.DataFrame:
    """
    金十数据-外汇-投机情绪报告
    外汇投机情绪报告显示当前市场多空仓位比例，数据由8家交易平台提供，涵盖11个主要货币对和1个黄金品种。
    报告内容: 品种: 澳元兑日元、澳元兑美元、欧元兑美元、欧元兑澳元、欧元兑日元、英镑兑美元、英镑兑日元、纽元兑美元、美元兑加元、美元兑瑞郎、美元兑日元以及现货黄金兑美元。
             数据: 由Shark - fx整合全球8家交易平台（ 包括 Oanda、 FXCM、 Insta、 Dukas、 MyFxBook以及FiboGroup） 的多空投机仓位数据而成。
    名词释义: 外汇投机情绪报告显示当前市场多空仓位比例，数据由8家交易平台提供，涵盖11个主要货币对和1个黄金品种。
    工具使用策略: Shark-fx声明表示，基于“主流通常都是错误的”的事实，当空头头寸超过60%，交易者就应该建立多头仓位； 同理，当市场多头头寸超过60%，交易者则应该建立空头仓位。此外，当多空仓位比例接近50%的情况下，我们则倾向于建议交易者不要进场，保持观望。
    https://datacenter.jin10.com/reportType/dc_ssi_trends
    :param start_date: 具体交易日
    :type start_date: str
    :param end_date: 具体交易日, 与 end_date 相同
    :type end_date: str
    :return: 投机情绪报告
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://datacenter-api.jin10.com/sentiment/datas"
    params = {
        "start_date": start_date,
        "end_date": end_date,
        "currency_pair": "",
    }
    headers = {
        "accept": "*/*",
        "accept-encoding": "",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "origin": "https://datacenter.jin10.com",
        "pragma": "no-cache",
        "referer": "https://datacenter.jin10.com/reportType/dc_ssi_trends",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/79.0.3945.130 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "",
        "x-version": "1.0.0",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["values"]).T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "date"}, inplace=True)
    for col in temp_df.columns[1:]:
        temp_df[col] = pd.to_numeric(temp_df[col], errors="coerce")
    return temp_df


if __name__ == "__main__":
    crypto_js_spot_df = crypto_js_spot()
    print(crypto_js_spot_df)

    test_date = datetime.now().date().isoformat().replace("-", "")

    macro_fx_sentiment_df = macro_fx_sentiment(start_date=test_date, end_date=test_date)
    print(macro_fx_sentiment_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/1 15:00
Desc: 同花顺-盈利预测
https://basic.10jqka.com.cn/new/600519/worth.html
"""

from io import StringIO

import pandas as pd
import requests

from akshare.utils.cons import headers


def stock_profit_forecast_ths(
    symbol: str = "600519", indicator: str = "预测年报每股收益"
) -> pd.DataFrame:
    """
    同花顺-盈利预测
    https://basic.10jqka.com.cn/new/600519/worth.html
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: choice of {"预测年报每股收益", "预测年报净利润", "业绩预测详表-机构", "业绩预测详表-详细指标预测"}
    :type indicator: str
    :return: 盈利预测
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/worth.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gbk"
    if "本年度暂无机构做出业绩预测" in r.text:
        # 处理 `本年度暂无机构做出业绩预测` 的情况
        if indicator == "预测年报每股收益":
            return pd.DataFrame()
        elif indicator == "预测年报净利润":
            return pd.DataFrame()
        elif indicator == "业绩预测详表-机构":
            temp_df = pd.read_html(StringIO(r.text))[0]
            columns_list = []
            for item in temp_df.columns:
                columns_list.append(item[1])
            columns_list[2] = "预测年报每股收益" + columns_list[2]
            columns_list[3] = "预测年报每股收益" + columns_list[3]
            columns_list[4] = "预测年报每股收益" + columns_list[4]
            columns_list[5] = "预测年报净利润" + columns_list[5]
            columns_list[6] = "预测年报净利润" + columns_list[6]
            columns_list[7] = "预测年报净利润" + columns_list[7]
            temp_df.columns = columns_list
            temp_df["报告日期"] = pd.to_datetime(
                temp_df["报告日期"], errors="coerce"
            ).dt.date
            return temp_df
        elif indicator == "业绩预测详表-详细指标预测":
            temp_df = pd.read_html(StringIO(r.text))[1]
            temp_df.columns = [
                item.replace("（", "-").replace("）", "") for item in temp_df.columns
            ]
            return temp_df
        else:
            return pd.DataFrame()
    else:
        if indicator == "预测年报每股收益":
            temp_df = pd.read_html(StringIO(r.text))[0]
            temp_df["年度"] = temp_df["年度"].astype(str)
            return temp_df
        elif indicator == "预测年报净利润":
            temp_df = pd.read_html(StringIO(r.text))[1]
            temp_df["年度"] = temp_df["年度"].astype(str)
            return temp_df
        elif indicator == "业绩预测详表-机构":
            temp_df = pd.read_html(StringIO(r.text))[2]
            columns_list = []
            for item in temp_df.columns:
                columns_list.append(item[1])
            columns_list[2] = "预测年报每股收益" + columns_list[2]
            columns_list[3] = "预测年报每股收益" + columns_list[3]
            columns_list[4] = "预测年报每股收益" + columns_list[4]
            columns_list[5] = "预测年报净利润" + columns_list[5]
            columns_list[6] = "预测年报净利润" + columns_list[6]
            columns_list[7] = "预测年报净利润" + columns_list[7]
            temp_df.columns = columns_list
            temp_df["报告日期"] = pd.to_datetime(temp_df["报告日期"]).dt.date
            return temp_df
        elif indicator == "业绩预测详表-详细指标预测":
            temp_df = pd.read_html(StringIO(r.text))[3]
            temp_df.columns = [
                item.replace("（", "-").replace("）", "") for item in temp_df.columns
            ]
            return temp_df
        else:
            return pd.DataFrame()


if __name__ == "__main__":
    stock_profit_forecast_ths_df = stock_profit_forecast_ths(
        symbol="600519", indicator="预测年报每股收益"
    )
    print(stock_profit_forecast_ths_df)

    stock_profit_forecast_ths_df = stock_profit_forecast_ths(
        symbol="600519", indicator="预测年报净利润"
    )
    print(stock_profit_forecast_ths_df)

    stock_profit_forecast_ths_df = stock_profit_forecast_ths(
        symbol="600519", indicator="业绩预测详表-机构"
    )
    print(stock_profit_forecast_ths_df)

    stock_profit_forecast_ths_df = stock_profit_forecast_ths(
        symbol="600519", indicator="业绩预测详表-详细指标预测"
    )
    print(stock_profit_forecast_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/11/26 18:00
Desc: 东方财富网-数据中心-估值分析-每日互动-每日互动-估值分析
https://data.eastmoney.com/gzfx/detail/300766.html
"""

import pandas as pd

from akshare.request import make_request_with_retry_json


def stock_value_em(symbol: str = "300766") -> pd.DataFrame:
    """
    东方财富网-数据中心-估值分析-每日互动-每日互动-估值分析
    https://data.eastmoney.com/gzfx/detail/300766.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 估值分析
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "TRADE_DATE",
        "sortTypes": "-1",
        "pageSize": "5000",
        "pageNumber": "1",
        "reportName": "RPT_VALUEANALYSIS_DET",
        "columns": "ALL",
        "quoteColumns": "",
        "source": "WEB",
        "client": "WEB",
        "filter": f'(SECURITY_CODE="{symbol}")',
    }
    data_json = make_request_with_retry_json(url, params=params)
    temp_json = data_json["result"]["data"]
    temp_df = pd.DataFrame(temp_json)
    temp_df.rename(
        columns={
            "TRADE_DATE": "数据日期",
            "CLOSE_PRICE": "当日收盘价",
            "CHANGE_RATE": "当日涨跌幅",
            "TOTAL_MARKET_CAP": "总市值",
            "NOTLIMITED_MARKETCAP_A": "流通市值",
            "TOTAL_SHARES": "总股本",
            "FREE_SHARES_A": "流通股本",
            "PE_TTM": "PE(TTM)",
            "PE_LAR": "PE(静)",
            "PB_MRQ": "市净率",
            "PEG_CAR": "PEG值",
            "PCF_OCF_TTM": "市现率",
            "PS_TTM": "市销率",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "数据日期",
            "当日收盘价",
            "当日涨跌幅",
            "总市值",
            "流通市值",
            "总股本",
            "流通股本",
            "PE(TTM)",
            "PE(静)",
            "市净率",
            "PEG值",
            "市现率",
            "市销率",
        ]
    ]
    temp_df["数据日期"] = pd.to_datetime(temp_df["数据日期"], errors="coerce").dt.date
    for item in temp_df.columns[1:]:
        temp_df[item] = pd.to_numeric(temp_df[item], errors="coerce")
    temp_df.sort_values(by="数据日期", ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    stock_value_em_df = stock_value_em(symbol="300766")
    print(stock_value_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/11/5 17:08
Desc: 东方财富-德国-经济数据
"""

import pandas as pd
import requests


def macro_germany_core(symbol: str = "EMG00179154") -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-宏观经济-德国-核心代码
    https://data.eastmoney.com/cjsj/foreign_1_0.html
    :param symbol: 代码
    :type symbol: str
    :return: 指定 symbol 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_GER",
        "columns": "ALL",
        "filter": f'(INDICATOR_ID="{symbol}")',
        "pageNumber": "1",
        "pageSize": "5000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "COUNTRY": "-",
            "INDICATOR_ID": "-",
            "INDICATOR_NAME": "-",
            "REPORT_DATE_CH": "时间",
            "REPORT_DATE": "-",
            "PUBLISH_DATE": "发布日期",
            "VALUE": "现值",
            "PRE_VALUE": "前值",
            "INDICATOR_IDOLD": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"])
    temp_df["现值"] = pd.to_numeric(temp_df["现值"])
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"]).dt.date
    temp_df.sort_values(["发布日期"], inplace=True, ignore_index=True)
    return temp_df


# 东方财富-德国-经济数据-IFO商业景气指数
def macro_germany_ifo() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-IFO商业景气指数
    https://data.eastmoney.com/cjsj/foreign_1_0.html
    :return: IFO商业景气指数
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00179154")
    return temp_df


# 东方财富-德国-经济数据-消费者物价指数月率终值
def macro_germany_cpi_monthly() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-消费者物价指数月率终值
    https://data.eastmoney.com/cjsj/foreign_1_1.html
    :return: 消费者物价指数月率终值
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00009758")
    return temp_df


# 东方财富-德国-经济数据-消费者物价指数年率终值
def macro_germany_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-消费者物价指数年率终值
    https://data.eastmoney.com/cjsj/foreign_1_2.html
    :return: 消费者物价指数年率终值
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00009756")
    return temp_df


# 东方财富-德国-经济数据-贸易帐(季调后)
def macro_germany_trade_adjusted() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-贸易帐(季调后)
    https://data.eastmoney.com/cjsj/foreign_1_3.html
    :return: 贸易帐(季调后)
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00009753")
    return temp_df


# 东方财富-德国-经济数据-GDP
def macro_germany_gdp() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-GDP
    https://data.eastmoney.com/cjsj/foreign_1_4.html
    :return: GDP
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00009720")
    return temp_df


# 东方财富-德国-经济数据-实际零售销售月率
def macro_germany_retail_sale_monthly() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-实际零售销售月率
    https://data.eastmoney.com/cjsj/foreign_1_5.html
    :return: 实际零售销售月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG01333186")
    return temp_df


# 东方财富-德国-经济数据-实际零售销售年率
def macro_germany_retail_sale_yearly() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-实际零售销售年率
    https://data.eastmoney.com/cjsj/foreign_1_6.html
    :return: 实际零售销售年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG01333192")
    return temp_df


# 东方财富-德国-经济数据-ZEW 经济景气指数
def macro_germany_zew() -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-德国-ZEW 经济景气指数
    https://data.eastmoney.com/cjsj/foreign_1_7.html
    :return: ZEW 经济景气指数
    :rtype: pandas.DataFrame
    """
    temp_df = macro_germany_core(symbol="EMG00172577")
    return temp_df


if __name__ == "__main__":
    macro_germany_ifo_df = macro_germany_ifo()
    print(macro_germany_ifo_df)

    macro_germany_cpi_monthly_df = macro_germany_cpi_monthly()
    print(macro_germany_cpi_monthly_df)

    macro_germany_cpi_yearly_df = macro_germany_cpi_yearly()
    print(macro_germany_cpi_yearly_df)

    macro_germany_trade_adjusted_df = macro_germany_trade_adjusted()
    print(macro_germany_trade_adjusted_df)

    macro_germany_gdp_df = macro_germany_gdp()
    print(macro_germany_gdp_df)

    macro_germany_retail_sale_monthly_df = macro_germany_retail_sale_monthly()
    print(macro_germany_retail_sale_monthly_df)

    macro_germany_retail_sale_yearly_df = macro_germany_retail_sale_yearly()
    print(macro_germany_retail_sale_yearly_df)

    macro_germany_zew_df = macro_germany_zew()
    print(macro_germany_zew_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/5 15:00
Desc: 九期网-商品期权手续费
https://www.9qihuo.com/qiquanshouxufei
"""

from functools import lru_cache
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


@lru_cache()
def option_comm_symbol() -> pd.DataFrame:
    import urllib3

    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    url = "https://www.9qihuo.com/qiquanshouxufei"
    r = requests.get(url, verify=False)
    soup = BeautifulSoup(r.text, features="lxml")
    name = [
        item.string.strip()
        for item in soup.find(name="div", attrs={"id": "inst_list"}).find_all(name="a")
    ]
    code = [
        item["href"].split("?")[1].split("=")[1]
        for item in soup.find(name="div", attrs={"id": "inst_list"}).find_all(name="a")
    ]
    temp_df = pd.DataFrame([name, code]).T
    temp_df.columns = ["品种名称", "品种代码"]
    return temp_df


def option_comm_info(symbol: str = "工业硅期权") -> pd.DataFrame:
    """
    九期网-商品期权手续费
    https://www.9qihuo.com/qiquanshouxufei
    :param symbol: choice of {"所有", "上海期货交易所", "大连商品交易所", "郑州商品交易所", "上海国际能源交易中心", "广州期货交易所"}
    :type symbol: str
    :return: 期权手续费
    :rtype: pandas.DataFrame
    """
    import urllib3

    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    symbol_df = option_comm_symbol()
    symbol_str = symbol_df[symbol_df["品种名称"].str.contains(symbol)][
        "品种代码"
    ].values[0]
    params = {"heyue": symbol_str}
    url = "https://www.9qihuo.com/qiquanshouxufei"
    r = requests.get(url, params=params, verify=False)
    temp_df = pd.read_html(StringIO(r.text))[0]
    market_symbol = temp_df.iloc[0, 0]
    columns = temp_df.iloc[2, :]
    temp_df = temp_df.iloc[3:, :]
    temp_df.columns = columns
    temp_df["交易所"] = market_symbol
    temp_df.reset_index(drop=True, inplace=True)
    temp_df.index.name = None
    temp_df.columns.name = None
    temp_df["现价"] = pd.to_numeric(temp_df["现价"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["每跳毛利/元"] = pd.to_numeric(temp_df["每跳毛利/元"], errors="coerce")
    temp_df["每跳净利/元"] = pd.to_numeric(temp_df["每跳净利/元"], errors="coerce")
    soup = BeautifulSoup(r.text, features="lxml")
    raw_date_text = soup.find(name="a", attrs={"id": "dlink"}).previous
    comm_update_time = raw_date_text.split("，")[0].strip("（手续费更新时间：")
    price_update_time = (
        raw_date_text.split("，")[1].strip("价格更新时间：").strip("。）")
    )
    temp_df["手续费更新时间"] = comm_update_time
    temp_df["价格更新时间"] = price_update_time
    return temp_df


if __name__ == "__main__":
    option_comm_symbol_df = option_comm_symbol()
    print(option_comm_symbol_df)

    option_comm_info_df = option_comm_info(symbol="工业硅期权")
    print(option_comm_info_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/12/24 16:20
Desc: 东方财富网-数据中心-新股申购-首发申报信息-首发申报企业信息
https://data.eastmoney.com/xg/xg/sbqy.html
"""

import pandas as pd
import requests

from akshare.utils.cons import headers
from akshare.utils.tqdm import get_tqdm


def stock_ipo_declare_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-首发申报企业信息
    https://data.eastmoney.com/xg/xg/sbqy.html
    :return: 首发申报企业信息
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "END_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_IPO_DECORGNEWEST",
        "columns": "DECLARE_ORG,STATE,REG_ADDRESS,RECOMMEND_ORG,LAW_FIRM,ACCOUNT_FIRM,IS_SUBMIT,"
                   "PREDICT_LISTING_MARKET,END_DATE,INFO_CODE,SECURITY_CODE,ORG_CODE,IS_REGISTER,"
                   "STATE_CODE,DERIVE_SECURITY_CODE,ORG_CODE_OLD,IS_STATE",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.reset_index(inplace=True)
    big_df["index"] = big_df.index + 1

    big_df.rename(
        columns={
            "index": "序号",
            "DECLARE_ORG": "企业名称",
            "STATE": "最新状态",
            "REG_ADDRESS": "注册地",
            "RECOMMEND_ORG": "保荐机构",
            "LAW_FIRM": "律师事务所",
            "ACCOUNT_FIRM": "会计师事务所",
            "PREDICT_LISTING_MARKET": "拟上市地点",
            "END_DATE": "更新日期",
            "INFO_CODE": "招股说明书",
        },
        inplace=True,
    )

    # 招股说明书链接处理（类似原有代码）
    if "招股说明书" in big_df.columns:
        big_df["招股说明书"] = [
            f"https://pdf.dfcfw.com/pdf/H2_{item}_1.pdf" if pd.notna(item) else ""
            for item in big_df["招股说明书"]
        ]

    big_df = big_df[
        [
            "序号",
            "企业名称",
            "最新状态",
            "注册地",
            "保荐机构",
            "律师事务所",
            "会计师事务所",
            "拟上市地点",
            "更新日期",
            # 可根据需要添加其他列，如 "IS_REGISTER" 等
            "招股说明书",
        ]
    ]

    # 日期处理
    big_df["更新日期"] = pd.to_datetime(big_df["更新日期"], errors="coerce", utc=True).dt.date
    return big_df


if __name__ == "__main__":
    stock_ipo_declare_em_df = stock_ipo_declare_em()
    print(stock_ipo_declare_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/9/12 16:50
Desc: 新浪财经-债券-可转债
https://money.finance.sina.com.cn/bond/info/sz128039.html
"""

from io import StringIO

import pandas as pd
import requests


def bond_cb_profile_sina(symbol: str = "sz128039") -> pd.DataFrame:
    """
    新浪财经-债券-可转债-详情资料
    https://money.finance.sina.com.cn/bond/info/sz128039.html
    :param symbol: 带市场标识的转债代码
    :type symbol: str
    :return: 可转债-详情资料
    :rtype: pandas.DataFrame
    """
    url = f"https://money.finance.sina.com.cn/bond/info/{symbol}.html"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = ["item", "value"]
    return temp_df


def bond_cb_summary_sina(symbol: str = "sh155255") -> pd.DataFrame:
    """
    新浪财经-债券-可转债-债券概况
    https://money.finance.sina.com.cn/bond/quotes/sh155255.html
    :param symbol: 带市场标识的转债代码
    :type symbol: str
    :return: 可转债-债券概况
    :rtype: pandas.DataFrame
    """
    url = f"https://money.finance.sina.com.cn/bond/quotes/{symbol}.html"
    r = requests.get(url)
    temp_df = pd.read_html(StringIO(r.text))[10]
    part1 = temp_df.iloc[:, 0:2].copy()
    part1.columns = ["item", "value"]
    part2 = temp_df.iloc[:, 2:4].copy()
    part2.columns = ["item", "value"]
    part3 = temp_df.iloc[:, 4:6].copy()
    part3.columns = ["item", "value"]
    big_df = pd.concat(objs=[part1, part2, part3], ignore_index=True)
    return big_df


if __name__ == "__main__":
    bond_cb_profile_sina_df = bond_cb_profile_sina(symbol="sz128039")
    print(bond_cb_profile_sina_df)

    bond_cb_summary_sina_df = bond_cb_summary_sina(symbol="sh155255")
    print(bond_cb_summary_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/8/4 14:00
Desc: 中证指数网站-指数列表
网站：https://www.csindex.com.cn/#/indices/family/list?index_series=1
"""
import warnings
from io import BytesIO

import pandas as pd
import requests


def index_csindex_all() -> pd.DataFrame:
    """
    中证指数网站-指数列表
    https://www.csindex.com.cn/#/indices/family/list?index_series=1
    Note: 但是不知道数据更新时间
    :return: 最新指数的列表,
    :rtype: pandas.DataFrame
    """
    warnings.filterwarnings("ignore", category=UserWarning, message="Workbook contains no default style")
    url = (
        f"https://www.csindex.com.cn/csindex-home/exportExcel/indexAll/CH"
    )

    headers = {
        "Content-Type": "application/json;charset=UTF-8",
    }
    playloads = {
        "sorter": {
            "sortField": "null",
            "sortOrder": None
        },
        "pager": {
            "pageNum": 1,
            "pageSize": 10
        },
        "indexFilter": {
            "ifCustomized": None,
            "ifTracked": None,
            "ifWeightCapped": None,
            "indexCompliance": None,
            "hotSpot": None,
            "indexClassify": None,
            "currency": None,
            "region": None,
            "indexSeries": ["1"],
            "undefined": None
        }
    }
    r = requests.post(url, json=playloads, headers=headers)

    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df["基日"] = pd.to_datetime(
        temp_df["基日"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["发布时间"] = pd.to_datetime(
        temp_df["发布时间"], format="%Y-%m-%d", errors="coerce"
    ).dt.date
    temp_df["指数代码"] = temp_df["指数代码"].astype(str).str.zfill(6)
    return temp_df


if __name__ == "__main__":
    index_csindex_all_df = index_csindex_all()
    print(index_csindex_all_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/21 17:00
Desc: 同花顺理财-基金数据-每日净值-ETF
https://fund.10jqka.com.cn/datacenter/jz/kfs/etf/
"""

import json

import pandas as pd
import requests


def fund_etf_spot_ths(date: str = "") -> pd.DataFrame:
    """
    同花顺理财-基金数据-每日净值-ETF-实时行情
    https://fund.10jqka.com.cn/datacenter/jz/kfs/etf/
    :param date: 查询日期
    :type date: str
    :return: ETF 实时行情
    :rtype: pandas.DataFrame
    """
    inner_date = "-".join([date[:4], date[4:6], date[6:]]) if date != "" else 0
    url = f"https://fund.10jqka.com.cn/data/Net/info/ETF_rate_desc_{inner_date}_0_1_9999_0_0_0_jsonp_g.html"
    r = requests.get(url, timeout=15)
    data_text = r.text[2:-1]
    data_json = json.loads(data_text)
    temp_df = pd.DataFrame(data_json["data"]["data"]).T
    temp_df.reset_index(inplace=True, drop=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"] + 1
    temp_df.rename(
        columns={
            "index": "序号",
            "code": "基金代码",
            "typename": "基金类型",
            "net": "当前-单位净值",
            "name": "基金名称",
            "totalnet": "当前-累计净值",
            "newnet": "最新-单位净值",
            "newtotalnet": "最新-累计净值",
            "newdate": "最新-交易日",
            "net1": "前一日-单位净值",
            "totalnet1": "前一日-累计净值",
            "ranges": "增长值",
            "rate": "增长率",
            "shstat": "赎回状态",
            "sgstat": "申购状态",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金名称",
            "当前-单位净值",
            "当前-累计净值",
            "前一日-单位净值",
            "前一日-累计净值",
            "增长值",
            "增长率",
            "赎回状态",
            "申购状态",
            "最新-交易日",
            "最新-单位净值",
            "最新-累计净值",
            "基金类型",
        ]
    ]
    query_date = inner_date if inner_date != 0 else temp_df["最新-交易日"][0]
    temp_df["查询日期"] = query_date
    temp_df["查询日期"] = pd.to_datetime(temp_df["查询日期"], errors="coerce").dt.date
    temp_df["当前-单位净值"] = pd.to_numeric(temp_df["当前-单位净值"], errors="coerce")
    temp_df["当前-累计净值"] = pd.to_numeric(temp_df["当前-累计净值"], errors="coerce")
    temp_df["前一日-单位净值"] = pd.to_numeric(
        temp_df["前一日-单位净值"], errors="coerce"
    )
    temp_df["前一日-累计净值"] = pd.to_numeric(
        temp_df["前一日-累计净值"], errors="coerce"
    )
    temp_df["增长值"] = pd.to_numeric(temp_df["增长值"], errors="coerce")
    temp_df["增长率"] = pd.to_numeric(temp_df["增长率"], errors="coerce")
    temp_df["最新-单位净值"] = pd.to_numeric(temp_df["最新-单位净值"], errors="coerce")
    temp_df["最新-累计净值"] = pd.to_numeric(temp_df["最新-累计净值"], errors="coerce")
    temp_df["最新-交易日"] = pd.to_datetime(
        temp_df["最新-交易日"], errors="coerce"
    ).dt.date
    return temp_df


if __name__ == "__main__":
    fund_etf_spot_ths_df = fund_etf_spot_ths(date="20240620")
    print(fund_etf_spot_ths_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/3/29 10:30
Desc: 同花顺-主营介绍
https://basic.10jqka.com.cn/new/000066/operate.html
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def stock_zyjs_ths(symbol: str = "000066") -> pd.DataFrame:
    """
    同花顺-主营介绍
    https://basic.10jqka.com.cn/new/000066/operate.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 主营介绍
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/operate.html"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/109.0.0.0 Safari/537.36"
    }
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, "lxml")
    content_list = [
        item.text.strip()
        for item in soup.find("ul", attrs={"class": "main_intro_list"}).find_all("li")
    ]
    columns_list = []
    value_list = []
    for item in content_list:
        columns_list.append(item.split("：")[0])
        value_list.append(
            item.split("：", maxsplit=1)[1]
            .replace("\t", "")
            .replace("\n", "")
            .replace(" ", "")
            .strip()
        )

    temp_df = pd.DataFrame(value_list, index=columns_list).T
    temp_df.insert(0, "股票代码", symbol)
    return temp_df


if __name__ == "__main__":
    stock_zyjs_ths_df = stock_zyjs_ths(symbol="000066")
    print(stock_zyjs_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/24 18:30
Desc: currencybeacon 提供的外汇数据
该网站需要先注册后获取 API 使用
https://currencyscoop.com/
"""

import pandas as pd
import requests


def currency_latest(
    base: str = "USD", symbols: str = "", api_key: str = ""
) -> pd.DataFrame:
    """
    Latest data from currencyscoop.com
    https://currencyscoop.com/api-documentation
    :param base: The base currency you would like to use for your rates
    :type base: str
    :param symbols: A list of currencies you will like to see the rates for. You can refer to a list all supported currencies here
    :type symbols: str
    :param api_key: Account -> Account Details -> API KEY (use as password in external tools)
    :type api_key: str
    :return: Latest data of base currency
    :rtype: pandas.DataFrame
    """
    params = {"base": base, "symbols": symbols, "api_key": api_key}
    url = "https://api.currencyscoop.com/v1/latest"
    r = requests.get(url, params=params)
    temp_df = pd.DataFrame.from_dict(r.json()["response"])
    temp_df["date"] = pd.to_datetime(temp_df["date"])
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "currency"}, inplace=True)
    return temp_df


def currency_history(
    base: str = "USD", date: str = "2023-02-03", symbols: str = "", api_key: str = ""
) -> pd.DataFrame:
    """
    Latest data from currencyscoop.com
    https://currencyscoop.com/api-documentation
    :param base: The base currency you would like to use for your rates
    :type base: str
    :param date: Specific date, e.g., "2020-02-03"
    :type date: str
    :param symbols: A list of currencies you will like to see the rates for. You can refer to a list all supported currencies here
    :type symbols: str
    :param api_key: Account -> Account Details -> API KEY (use as password in external tools)
    :type api_key: str
    :return: Latest data of base currency
    :rtype: pandas.DataFrame
    """
    params = {"base": base, "date": date, "symbols": symbols, "api_key": api_key}
    url = "https://api.currencyscoop.com/v1/historical"
    r = requests.get(url, params=params)
    temp_df = pd.DataFrame.from_dict(r.json()["response"])
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "currency"}, inplace=True)
    return temp_df


def currency_time_series(
    base: str = "USD",
    start_date: str = "2023-02-03",
    end_date: str = "2023-03-04",
    symbols: str = "",
    api_key: str = "",
) -> pd.DataFrame:
    """
    Time-series data from currencyscoop.com
    P.S. need special authority
    https://currencyscoop.com/api-documentation
    :param base: The base currency you would like to use for your rates
    :type base: str
    :param start_date: Specific date, e.g., "2020-02-03"
    :type start_date: str
    :param end_date: Specific date, e.g., "2020-02-03"
    :type end_date: str
    :param symbols: A list of currencies you will like to see the rates for. You can refer to a list all supported currencies here
    :type symbols: str
    :param api_key: Account -> Account Details -> API KEY (use as password in external tools)
    :type api_key: str
    :return: Latest data of base currency
    :rtype: pandas.DataFrame
    """
    params = {
        "base": base,
        "api_key": api_key,
        "start_date": start_date,
        "end_date": end_date,
        "symbols": symbols,
    }
    url = "https://api.currencyscoop.com/v1/timeseries"
    r = requests.get(url, params=params)
    temp_df = pd.DataFrame.from_dict(r.json()["response"])
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "date"}, inplace=True)
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    return temp_df


def currency_currencies(c_type: str = "fiat", api_key: str = "") -> pd.DataFrame:
    """
    currencies data from currencyscoop.com
    https://currencyscoop.com/api-documentation
    :param c_type: now only "fiat" can return data
    :type c_type: str
    :param api_key: Account -> Account Details -> API KEY (use as password in external tools)
    :type api_key: str
    :return: Latest data of base currency
    :rtype: pandas.DataFrame
    """
    params = {"type": c_type, "api_key": api_key}
    url = "https://api.currencyscoop.com/v1/currencies"
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["response"])
    return temp_df


def currency_convert(
    base: str = "USD",
    to: str = "CNY",
    amount: str = "10000",
    api_key: str = "",
) -> pd.DataFrame:
    """
    currencies data from currencyscoop.com
    https://currencyscoop.com/api-documentation
    :param base: The base currency you would like to use for your rates
    :type base: str
    :param to: The currency you would like to use for your rates
    :type to: str
    :param amount: The amount of base currency
    :type amount: str
    :param api_key: Account -> Account Details -> API KEY (use as password in external tools)
    :type api_key: str
    :return: Latest data of base currency
    :rtype: pandas.Series
    """
    params = {
        "from": base,
        "to": to,
        "amount": amount,
        "api_key": api_key,
    }
    url = "https://api.currencyscoop.com/v1/convert"
    r = requests.get(url, params=params)
    temp_se = pd.Series(r.json()["response"])
    temp_se["timestamp"] = pd.to_datetime(temp_se["timestamp"], unit="s")
    temp_df = temp_se.to_frame()
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["item", "value"]
    return temp_df


if __name__ == "__main__":
    currency_latest_df = currency_latest(base="USD", api_key="")
    print(currency_latest_df)

    currency_history_df = currency_history(base="USD", date="2023-02-03", api_key="")
    print(currency_history_df)

    currency_time_series_df = currency_time_series(
        base="USD",
        start_date="2023-02-03",
        end_date="2023-03-04",
        symbols="",
        api_key="",
    )
    print(currency_time_series_df)

    currency_currencies_df = currency_currencies(c_type="fiat", api_key="")
    print(currency_currencies_df)

    currency_convert_se = currency_convert(
        base="USD", to="CNY", amount="10000", api_key=""
    )
    print(currency_convert_se)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/10 00:00
Desc: 新浪财经-龙虎榜
https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/lhb/index.phtml
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.tqdm import get_tqdm


def stock_lhb_detail_daily_sina(date: str = "20240222") -> pd.DataFrame:
    """
    龙虎榜-每日详情
    https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/lhb/index.phtml
    :param date: 交易日
    :type date: str
    :return: 龙虎榜-每日详情
    :rtype: pandas.DataFrame
    """
    date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://vip.stock.finance.sina.com.cn/q/go.php/vInvestConsult/kind/lhb/index.phtml"
    params = {"tradedate": date}
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, features="lxml")
    selected_html = soup.find(name="div", attrs={"class": "list"}).find_all(
        name="table", attrs={"class": "list_table"}
    )
    big_df = pd.DataFrame()
    for table in selected_html:
        temp_df = pd.read_html(StringIO(table.prettify()), header=0, skiprows=1)[0]
        temp_symbol = pd.read_html(StringIO(table.prettify()))[0].iat[0, 0]
        temp_df["指标"] = temp_symbol
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    del big_df["查看详情"]
    big_df.columns = [
        "序号",
        "股票代码",
        "股票名称",
        "收盘价",
        "对应值",
        "成交量",
        "成交额",
        "指标",
    ]
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["对应值"] = pd.to_numeric(big_df["对应值"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    return big_df


def _find_last_page(
    url: str = "https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/ggtj/index.phtml",
    recent_day: str = "60",
):
    params = {
        "last": recent_day,
        "p": "1",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, "lxml")
    try:
        previous_page = int(soup.find_all(attrs={"class": "page"})[-2].text)
    except:  # noqa: E722
        previous_page = 1
    if previous_page != 1:
        while True:
            params = {
                "last": recent_day,
                "p": previous_page,
            }
            r = requests.get(url, params=params)
            soup = BeautifulSoup(r.text, features="lxml")
            last_page = int(soup.find_all(attrs={"class": "page"})[-2].text)
            if last_page != previous_page:
                previous_page = last_page
                continue
            else:
                break
    return previous_page


def stock_lhb_ggtj_sina(symbol: str = "5") -> pd.DataFrame:
    """
    龙虎榜-个股上榜统计
    https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/ggtj/index.phtml
    :param symbol: choice of {"5": 最近 5 天; "10": 最近 10 天; "30": 最近 30 天; "60": 最近 60 天;}
    :type symbol: str
    :return: 龙虎榜-个股上榜统计
    :rtype: pandas.DataFrame
    """
    url = (
        "https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/ggtj/index.phtml"
    )
    last_page_num = _find_last_page(url, symbol)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, last_page_num + 1), leave=False):
        params = {
            "last": symbol,
            "p": page,
        }
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[0:, :]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df.columns = [
        "股票代码",
        "股票名称",
        "上榜次数",
        "累积购买额",
        "累积卖出额",
        "净额",
        "买入席位数",
        "卖出席位数",
    ]
    return big_df


def stock_lhb_yytj_sina(symbol: str = "5") -> pd.DataFrame:
    """
    龙虎榜-营业部上榜统计
    https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/yytj/index.phtml
    :param symbol: choice of {"5": 最近 5 天; "10": 最近 10 天; "30": 最近 30 天; "60": 最近 60 天;}
    :type symbol: str
    :return: 龙虎榜-营业部上榜统计
    :rtype: pandas.DataFrame
    """
    url = (
        "https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/yytj/index.phtml"
    )
    last_page_num = _find_last_page(url, symbol)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, last_page_num + 1), leave=False):
        params = {
            "last": "5",
            "p": page,
        }
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[0:, :]
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "营业部名称",
        "上榜次数",
        "累积购买额",
        "买入席位数",
        "累积卖出额",
        "卖出席位数",
        "买入前三股票",
    ]
    big_df["上榜次数"] = pd.to_numeric(big_df["上榜次数"], errors="coerce")
    big_df["买入席位数"] = pd.to_numeric(big_df["买入席位数"], errors="coerce")
    big_df["卖出席位数"] = pd.to_numeric(big_df["卖出席位数"], errors="coerce")
    return big_df


def stock_lhb_jgzz_sina(symbol: str = "5") -> pd.DataFrame:
    """
    龙虎榜-机构席位追踪
    https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgzz/index.phtml
    :param symbol: choice of {"5": 最近 5 天; "10": 最近 10 天; "30": 最近 30 天; "60": 最近 60 天;}
    :type symbol: str
    :return: 龙虎榜-机构席位追踪
    :rtype: pandas.DataFrame
    """
    url = (
        "https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgzz/index.phtml"
    )
    last_page_num = _find_last_page(url, symbol)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, last_page_num + 1), leave=False):
        params = {
            "last": symbol,
            "p": page,
        }
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[0:, :]
        if temp_df.empty:
            continue
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    del big_df["当前价"]
    del big_df["涨跌幅"]
    big_df.columns = [
        "股票代码",
        "股票名称",
        "累积买入额",
        "买入次数",
        "累积卖出额",
        "卖出次数",
        "净额",
    ]
    big_df["买入次数"] = pd.to_numeric(big_df["买入次数"], errors="coerce")
    big_df["卖出次数"] = pd.to_numeric(big_df["卖出次数"], errors="coerce")
    return big_df


def stock_lhb_jgmx_sina() -> pd.DataFrame:
    """
    龙虎榜-机构席位成交明细
    https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgmx/index.phtml
    :return: 龙虎榜-机构席位成交明细
    :rtype: pandas.DataFrame
    """
    url = (
        "https://vip.stock.finance.sina.com.cn/q/go.php/vLHBData/kind/jgmx/index.phtml"
    )
    params = {
        "p": "1",
    }
    r = requests.get(url, params=params)
    soup = BeautifulSoup(r.text, features="lxml")
    try:
        last_page_num = int(soup.find_all(attrs={"class": "page"})[-2].text)
    except:  # noqa: E722
        last_page_num = 1
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, last_page_num + 1), leave=False):
        params = {
            "p": page,
        }
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text))[0].iloc[0:, :]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["股票代码"] = big_df["股票代码"].astype(str).str.zfill(6)
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    big_df.rename(
        columns={
            "机构席位买入额(万)": "机构席位买入额",
            "机构席位卖出额(万)": "机构席位卖出额",
        },
        inplace=True,
    )
    big_df["机构席位买入额"] = pd.to_numeric(big_df["机构席位买入额"], errors="coerce")
    big_df["机构席位卖出额"] = pd.to_numeric(big_df["机构席位卖出额"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_lhb_detail_daily_sina_df = stock_lhb_detail_daily_sina(date="20240222")
    print(stock_lhb_detail_daily_sina_df)

    stock_lhb_ggtj_sina_df = stock_lhb_ggtj_sina(symbol="5")
    print(stock_lhb_ggtj_sina_df)

    stock_lhb_yytj_sina_df = stock_lhb_yytj_sina(symbol="5")
    print(stock_lhb_yytj_sina_df)

    stock_lhb_jgzz_sina_df = stock_lhb_jgzz_sina(symbol="5")
    print(stock_lhb_jgzz_sina_df)

    stock_lhb_jgmx_sina_df = stock_lhb_jgmx_sina()
    print(stock_lhb_jgmx_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/20 17:00
Desc: 东方财富网-期货行情
https://qhweb.eastmoney.com/quote
"""

import re
from functools import lru_cache
from typing import Tuple, Dict

import pandas as pd
import requests


def __futures_hist_separate_char_and_numbers_em(symbol: str = "焦煤2506") -> tuple:
    """
    东方财富网-期货行情-交易所品种对照表原始数据
    https://quote.eastmoney.com/qihuo/al2505.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 交易所品种对照表原始数据
    :rtype: pandas.DataFrame
    """
    char = re.findall(pattern="[\u4e00-\u9fa5a-zA-Z]+", string=symbol)
    numbers = re.findall(pattern=r"\d+", string=symbol)
    return char[0], numbers[0]


@lru_cache()
def __fetch_exchange_symbol_raw_em() -> list:
    """
    东方财富网-期货行情-交易所品种对照表原始数据
    https://quote.eastmoney.com/qihuo/al2505.html
    :return: 交易所品种对照表原始数据
    :rtype: pandas.DataFrame
    """
    url = "https://futsse-static.eastmoney.com/redis"
    params = {"msgid": "gnweb"}
    r = requests.get(url, params=params)
    data_json = r.json()
    all_exchange_symbol_list = []
    for item in data_json:
        params = {"msgid": str(item["mktid"])}
        r = requests.get(url, params=params)
        inner_data_json = r.json()
        for num in range(1, len(inner_data_json) + 1):
            params = {"msgid": str(item["mktid"]) + f"_{num}"}
            r = requests.get(url, params=params)
            inner_data_json = r.json()
            all_exchange_symbol_list.extend(inner_data_json)
    return all_exchange_symbol_list


@lru_cache()
def __get_exchange_symbol_map() -> Tuple[Dict, Dict, Dict, Dict]:
    """
    东方财富网-期货行情-交易所品种映射
    https://quote.eastmoney.com/qihuo/al2505.html
    :return: 交易所品种映射
    :rtype: pandas.DataFrame
    """
    all_exchange_symbol_list = __fetch_exchange_symbol_raw_em()
    c_contract_mkt = {}
    c_contract_to_e_contract = {}
    e_symbol_mkt = {}
    c_symbol_mkt = {}
    for item in all_exchange_symbol_list:
        c_contract_mkt[item["name"]] = item["mktid"]
        c_contract_to_e_contract[item["name"]] = item["code"]
        e_symbol_mkt[item["vcode"]] = item["mktid"]
        c_symbol_mkt[item["vname"]] = item["mktid"]
    return c_contract_mkt, c_contract_to_e_contract, e_symbol_mkt, c_symbol_mkt


def futures_hist_table_em() -> pd.DataFrame:
    """
    东方财富网-期货行情-交易所品种对照表
    https://quote.eastmoney.com/qihuo/al2505.html
    :return: 交易所品种对照表
    :rtype: pandas.DataFrame
    """
    all_exchange_symbol_list = __fetch_exchange_symbol_raw_em()
    temp_df = pd.DataFrame(all_exchange_symbol_list)
    temp_df = temp_df[["mktname", "name", "code"]]
    temp_df.columns = ["市场简称", "合约中文代码", "合约代码"]
    return temp_df


def futures_hist_em(
    symbol: str = "热卷主连",
    period: str = "daily",
    start_date: str = "19900101",
    end_date: str = "20500101",
) -> pd.DataFrame:
    """
    东方财富网-期货行情-行情数据
    https://qhweb.eastmoney.com/quote
    :param symbol: 期货代码
    :type symbol: str
    :param period: choice of {'daily', 'weekly', 'monthly'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    period_dict = {"daily": "101", "weekly": "102", "monthly": "103"}
    c_contract_mkt, c_contract_to_e_contract, e_symbol_mkt, c_symbol_mkt = (
        __get_exchange_symbol_map()
    )
    try:
        sec_id = f"{c_contract_mkt[symbol]}.{c_contract_to_e_contract[symbol]}"
    except KeyError:
        symbol_char, numbers = __futures_hist_separate_char_and_numbers_em(symbol)
        if re.match(pattern="^[\u4e00-\u9fa5]+$", string=symbol_char):
            sec_id = str(c_symbol_mkt[symbol_char]) + "." + symbol
        else:
            sec_id = str(e_symbol_mkt[symbol_char]) + "." + symbol
    params = {
        "secid": sec_id,
        "klt": period_dict[period],
        "fqt": "1",
        "lmt": "10000",
        "end": "20500000",
        "iscca": "1",
        "fields1": "f1,f2,f3,f4,f5,f6,f7,f8",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f62,f63,f64",
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "forcect": "1",
    }
    r = requests.get(url, timeout=15, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    if temp_df.empty: return temp_df
    temp_df.columns = [
        "时间",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "-",
        "涨跌幅",
        "涨跌",
        "_",
        "_",
        "持仓量",
        "_",
    ]
    temp_df = temp_df[
        [
            "时间",
            "开盘",
            "最高",
            "最低",
            "收盘",
            "涨跌",
            "涨跌幅",
            "成交量",
            "成交额",
            "持仓量",
        ]
    ]
    temp_df.index = pd.to_datetime(temp_df["时间"])
    temp_df = temp_df[start_date:end_date]
    temp_df.reset_index(drop=True, inplace=True)
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
    temp_df["时间"] = pd.to_datetime(temp_df["时间"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    futures_hist_table_em_df = futures_hist_table_em()
    print(futures_hist_table_em_df)

    futures_hist_em_df = futures_hist_em(symbol="热卷主连", period="daily")
    print(futures_hist_em_df)
